@@ -24,20 +24,20 @@
 #define LXWEATHER_HTTPUTIL_HEADER
 
 #include <glib.h>
-
-static const gint HTTP_STATUS_OK = 200;
+#include <curl/curl.h>
 
 /**
  * Returns the contents of the requested URL
  *
  * @param pczURL The URL to retrieve [in].
- * @param piRetCode The return code supplied with the response [out].
+ * @param pcData A pointer to a null-terminated buffer containing the textual
+ *         representation of the response. Must be freed by the caller. [out].
  * @param piDataSize The resulting data length [out].
+ * @param headers Extra headers for GET request [in].
  *
- * @return A pointer to a null-terminated buffer containing the textual
- *         representation of the response. Must be freed by the caller.
+ * @return The return code supplied by CURL
  */
-gpointer
-getURL(const gchar * pczURL, gint * piRetCode, gint * piDataSize);
+CURLcode
+getURL(const gchar * pczURL, gchar ** pcData, gint * piDataSize, const gchar ** headers);
 
 #endif
